#if !defined( _complex_h )
#define _complex_h

#include "JObject.h"

class
#include "JBase.hpp"
complex {
  public:
    double r, i;
    double norm() const { return r*r+i*i;}
    double abs() const;
    double arg() const;
    complex(double _r = 0, double _i = 0) : r(_r), i(_i) {}
    complex(const complex& c) : r(c.r), i(c.i) {}
    complex& operator=(const complex& c) { r=c.r; i=c.i; return *this;}
    complex& operator+=(const complex& c) { r+=c.r; i+=c.i; return *this;}
    complex& operator-=(const complex& c) { r-=c.r; i-=c.i; return *this;}
    complex& operator*=(const complex& c);
    complex& operator/=(const complex& c);
    complex& operator=(double _r) { r=_r; i=0; return *this;}
    complex& operator+=(double _r) { r+=_r; return *this;}
    complex& operator-=(double _r) { r-=_r; return *this;}
    complex& operator*=(double _r) { r*=_r; i*=_r; return *this;} 
    complex& operator/=(double _r) { r/=_r; i/=_r; return *this;}
    complex operator+(const complex& c) { return complex(r+c.r, i+c.i);}
    complex operator-(const complex& c) { return complex(r-c.r, i-c.i);}
    complex operator*(const complex& c);
    complex operator/(const complex& c);
    boolean operator==(const complex& c) { return (r == c.r) && (i == c.i);}
    boolean operator!=(const complex& c) { return (r != c.r) || (i != c.i);}
    friend complex operator-(const complex& c) { return complex(-c.r, -c.i);}
    friend complex operator+(const complex& c, double _r) { return complex(c.r+_r, c.i);}
    friend complex operator+(double _r, const complex& c) { return complex(c.r+_r, c.i);}
    friend complex operator-(const complex& c, double _r) { return complex(c.r-_r, c.i);}
    friend complex operator-(double _r, const complex& c) { return complex(c.r-_r, c.i);}
    friend complex operator*(const complex& c, double _r) { return complex(c.r*_r, c.i*_r);}
    friend complex operator*(double _r, const complex& c) { return complex(c.r*_r, c.i*_r);}
    friend complex operator/(const complex& c, double _r) { return complex(c.r/_r, c.i/_r);}
    friend complex operator/(double _r, const complex& c) { return complex(c.r, c.i);}
    friend boolean operator==(const complex& c, double _r) { return (c.r == _r) && (c.i == 0.0);}
    friend boolean operator==(double _r, const complex& c) { return (c.r == _r) && (c.i == 0.0);}
    friend boolean operator!=(const complex& c, double _r) { return (c.r != _r) || (c.i != 0.0);}
    friend boolean operator!=(double _r, const complex& c) { return (c.r != _r) || (c.i != 0.0);}
};

#endif
